
      PROGRAM LMDERTEST
C     **********
C
C     THIS PROGRAM TESTS CODES FOR THE LEAST-SQUARES SOLUTION OF
C     M NONLINEAR EQUATIONS IN N VARIABLES. IT CONSISTS OF A DRIVER
C     AND AN INTERFACE SUBROUTINE FCN. THE DRIVER READS IN DATA,
C     CALLS THE NONLINEAR LEAST-SQUARES SOLVER, AND FINALLY PRINTS
C     OUT INFORMATION ON THE PERFORMANCE OF THE SOLVER. THIS IS
C     ONLY A SAMPLE DRIVER, MANY OTHER DRIVERS ARE POSSIBLE. THE
C     INTERFACE SUBROUTINE FCN IS NECESSARY TO TAKE INTO ACCOUNT THE
C     FORMS OF CALLING SEQUENCES USED BY THE FUNCTION AND JACOBIAN
C     SUBROUTINES IN THE VARIOUS NONLINEAR LEAST-SQUARES SOLVERS.
C
C     SUBPROGRAMS CALLED
C
C       USER-SUPPLIED ...... FCN
C
C       MINPACK-SUPPLIED ... SPMPAR,ENORM,INITPT,LMDER1,SSQFCN
C
C       FORTRAN-SUPPLIED ... SQRT
C
C     ARGONNE NATIONAL LABORATORY. MINPACK PROJECT. MARCH 1980.
C     BURTON S. GARBOW, KENNETH E. HILLSTROM, JORGE J. MORE
C
C     **********
      IMPLICIT NONE

c     .. Constants ..
      INTEGER   NMAX, MMAX, MAXTRIES, NPROB, NREAD, NWRITE
      PARAMETER ( NMAX = 40, MAXTRIES = 2, NPROB = 14, MMAX = 236)
      PARAMETER ( NREAD= 5, NWRITE = 6 )

      INTEGER PROBLEMS(NPROB)
      DATA PROBLEMS(1:NPROB) /
     * 2, 1, 3, 4, 9, 5, 6, 7, 8, 10, 11, 12, 13, 14 /

      COMMON /COUNTER/ IPROB

c     .. Local Scalars ..
      INTEGER I, IC, INFO, K, LDFJAC, M, N, NFEV, NJEV, IPROB,
     *        MAXFEV, MODE, NPRINT, IFLAG, P

      DOUBLE PRECISION FACTOR, FNORM1, FNORM2, FTOL, XTOL, GTOL

c     .. Local Arrays ..
      INTEGER MA(MAXTRIES), NA(MAXTRIES), NF(MAXTRIES), NJ(MAXTRIES),
     *        NP(MAXTRIES), NX(MAXTRIES), IPVT(NMAX)

      CHARACTER*64 PNAME

      DOUBLE PRECISION FJAC(MMAX, NMAX), FNM(MAXTRIES), FVEC(MMAX),
     *                  X(NMAX), DIAG(NMAX), QTF(NMAX), WA1(NMAX),
     *                  WA2(NMAX), WA3(NMAX), WA4(MMAX)
     
c     .. External Subroutines ..
      DOUBLE PRECISION ENORM
      
      EXTERNAL FCN
C
C     LOGICAL INPUT UNIT IS ASSUMED TO BE NUMBER 5.
C     LOGICAL OUTPUT UNIT IS ASSUMED TO BE NUMBER 6.
C

      LDFJAC = MMAX
      IFLAG= 1

C     Loop over all problems
      DO 30 P = 1, NPROB
        IC = 0
        DO 20 K = 1, MAXTRIES
          IPROB= PROBLEMS(P)
          CALL INITPT(N, M, X, IPROB, K, FACTOR,FTOL, XTOL, GTOL, MAXFEV,
     *                MODE, PNAME)
C         N=0 signals last try
          IF (N.EQ.0) GOTO 25

C         Calculate norm of starting vector
C         CALL SSQFCN(M, N, X, FVEC, IPROB)
          CALL FCN(M, N, X, FVEC, FJAC, LDFJAC, IFLAG)

          FNORM1 = ENORM(M,FVEC)

C         Fit data
          WRITE (*, 60) P, TRIM(PNAME), N, M
          CALL LMDER (FCN,M,N,X,FVEC,FJAC,LDFJAC,FTOL,XTOL,GTOL,
     *               MAXFEV,DIAG,MODE,FACTOR,NPRINT,INFO,NFEV,NJEV,
     *               IPVT,QTF,WA1,WA2,WA3,WA4)

C         Calculate norm of result
C          CALL SSQFCN(M,N,X,FVEC,IPROB)
          CALL FCN(M, N, X, FVEC, FJAC, LDFJAC, IFLAG)
          FNORM2 = ENORM(M,FVEC)

          IC = IC + 1
          NP(IC) = IPROB
          NA(IC) = N
          MA(IC) = M
          NF(IC) = NFEV
          NJ(IC) = NJEV
          NX(IC) = INFO
          FNM(IC) = FNORM2
          WRITE (NWRITE, 70)
     *          FNORM1,FNORM2,NFEV,NJEV,INFO,(X(I), I = 1, N)
   20   CONTINUE
   25   CONTINUE
        IF( IC.EQ.0) GOTO 30
        WRITE (*, 80) IC
        WRITE (*, 90)
        DO 40 I = 1, IC
          WRITE (*, 100) NP(I),NA(I),MA(I),NF(I),NJ(I),NX(I),FNM(I)
   40   CONTINUE

   30 CONTINUE
      STOP
   60 FORMAT (//// 5X, 8H PROBLEM, I5,": ", A // 5X,
     *        11H DIMENSIONS, 2I5, 5X //  )
   70 FORMAT (5X, 33H INITIAL L2 NORM OF THE RESIDUALS, ES15.7 // 5X,
     *        33H FINAL L2 NORM OF THE RESIDUALS  , ES15.7 // 5X,
     *        33H NUMBER OF FUNCTION EVALUATIONS  , I10 // 5X,
     *        33H NUMBER OF JACOBIAN EVALUATIONS  , I10 // 5X,
     *        15H EXIT PARAMETER, 18X, I10 // 5X,
     *        27H FINAL APPROXIMATE SOLUTION // (4X, 5ES15.7))
   80 FORMAT (11H SUMMARY OF, I3, 16H CALLS TO LMDER1 /)
   90 FORMAT (49H IPROB   N    M   NFEV  NJEV  INFO  FINAL L2 NORM /)
  100 FORMAT (3I5, 3I6, 2X, ES15.7)
      END




      SUBROUTINE FCN(M, N, X, FVEC, FJAC, LDFJAC, IFLAG)

      IMPLICIT NONE

      INTEGER M,N,LDFJAC,IFLAG
      DOUBLE PRECISION X(N),FVEC(M),FJAC(LDFJAC,M)
C     **********
C
C     THE CALLING SEQUENCE OF FCN SHOULD BE IDENTICAL TO THE
C     CALLING SEQUENCE OF THE FUNCTION SUBROUTINE IN THE NONLINEAR
C     LEAST-SQUARES SOLVER. FCN SHOULD ONLY CALL THE TESTING
C     FUNCTION AND JACOBIAN SUBROUTINES SSQFCN AND SSQJAC WITH
C     THE APPROPRIATE VALUE OF PROBLEM NUMBER (IPROB).
C
C     SUBPROGRAMS CALLED
C
C       MINPACK-SUPPLIED ... SSQFCN,SSQJAC
C
C     ARGONNE NATIONAL LABORATORY. MINPACK PROJECT. MARCH 1980.
C     BURTON S. GARBOW, KENNETH E. HILLSTROM, JORGE J. MORE
C
C     **********

      INTEGER IPROB, NPROB, MMAX
      PARAMETER ( NPROB = 14, MMAX = 236 )

      COMMON /COUNTER/ IPROB

      DOUBLE PRECISION XDATA(MMAX, NPROB), YDATA(MMAX,NPROB)

c                                Problem 1
C     NIST CHWIRUT2
C     See http://www.itl.nist.gov/div898/strd/nls/data/chwirut2.shtml.
C     x-data 
      DATA ( XDATA(M, 1), M=1, 54) /
     *  0.500D0, 1.000D0, 1.750D0, 3.750D0, 5.750D0, 0.875D0, 2.250D0,
     *  3.250D0, 5.250D0, 0.750D0, 1.750D0, 2.750D0, 4.750D0, 0.625D0,
     *  1.250D0, 2.250D0, 4.250D0, 0.500D0, 3.000D0, 0.750D0, 3.000D0,
     *  1.500D0, 6.000D0, 3.000D0, 6.000D0, 1.500D0, 3.000D0, 0.500D0,
     *  2.000D0, 4.000D0, 0.750D0, 2.000D0, 5.000D0, 0.750D0, 2.250D0,
     *  3.750D0, 5.750D0, 3.000D0, 0.750D0, 2.500D0, 4.000D0, 0.750D0,
     *  2.500D0, 4.000D0, 0.750D0, 2.500D0, 4.000D0, 0.500D0, 6.000D0,
     *  3.000D0, 0.500D0, 2.750D0, 0.500D0, 1.750D0 /
C     y-data 
      DATA ( YDATA(M, 1), M=1, 54) /
     *  92.9000D0, 57.1000D0, 31.0500D0, 11.5875D0, 8.0250D0, 63.6000D0,
     *  21.4000D0, 14.2500D0,  8.4750D0, 63.8000D0, 26.8000D0, 16.4625D0,
     *   7.1250D0, 67.3000D0, 41.0000D0, 21.1500D0,  8.1750D0, 81.5000D0,
     *  13.1200D0, 59.9000D0, 14.6200D0, 32.9000D0,  5.4400D0, 12.5600D0,
     *   5.4400D0, 32.0000D0, 13.9500D0, 75.8000D0, 20.0000D0, 10.4200D0,
     *  59.5000D0, 21.6700D0,  8.5500D0, 62.0000D0, 20.2000D0,  7.7600D0,
     *   3.7500D0, 11.8100D0, 54.7000D0, 23.7000D0, 11.5500D0, 61.3000D0,
     *  17.7000D0,  8.7400D0, 59.2000D0, 16.3000D0,  8.6200D0, 81.0000D0,
     *   4.8700D0, 14.6200D0, 81.7000D0, 17.1700D0, 81.3000D0, 28.9000D0 /
C
C                                Problem 2
C     NIST Misra 1a
C     See http://www.itl.nist.gov/div898/strd/nls/data/misra1a.shtml.
C     x-data 
      DATA ( XDATA(M, 2), M=1, 14) /
     *  77.6D0, 114.9D0, 141.1D0, 190.8D0, 239.9D0, 289.0D0, 332.8D0,
     * 378.4D0, 434.8D0, 477.3D0, 536.8D0, 593.1D0, 689.1D0, 760.0D0 /
C     y-data 
      DATA ( YDATA(M, 2), M=1, 14) /
     * 10.07D0, 14.73D0, 17.94D0, 23.93D0, 29.61D0, 35.18D0, 40.02D0,
     * 44.82D0, 50.76D0, 55.05D0, 61.01D0, 66.40D0, 75.47D0, 81.78D0 /
C
C                                Problem 3
C     NIST Hahn 1
C     See http://www.itl.nist.gov/div898/strd/nls/data/hahn1.shtml.
C     x-data 
      DATA ( XDATA(M, 3), M=1, 236) /
     *  24.41D0,  34.82D0,  44.09D0,  45.07D0,  54.98D0,  65.51D0,
     *  70.53D0,  75.70D0,  89.57D0,  91.14D0,  96.40D0,  97.19D0,
     * 114.26D0, 120.25D0, 127.08D0, 133.55D0, 133.61D0, 158.67D0,
     * 172.74D0, 171.31D0, 202.14D0, 220.55D0, 221.05D0, 221.39D0,
     * 250.99D0, 268.99D0, 271.80D0, 271.97D0, 321.31D0, 321.69D0,
     * 330.14D0, 333.03D0, 333.47D0, 340.77D0, 345.65D0, 373.11D0,
     * 373.79D0, 411.82D0, 419.51D0, 421.59D0, 422.02D0, 422.47D0,
     * 422.61D0, 441.75D0, 447.41D0, 448.7D0 , 472.89D0, 476.69D0,
     * 522.47D0, 522.62D0, 524.43D0, 546.75D0, 549.53D0, 575.29D0,
     * 576.00D0, 625.55D0,  20.15D0,  28.78D0,  29.57D0,  37.41D0,
     *  39.12D0,  50.24D0,  61.38D0,  66.25D0,  73.42D0,  95.52D0,
     * 107.32D0, 122.04D0, 134.03D0, 163.19D0, 163.48D0, 175.70D0,
     * 179.86D0, 211.27D0, 217.78D0, 219.14D0, 262.52D0, 268.01D0,
     * 268.62D0, 336.25D0, 337.23D0, 339.33D0, 427.38D0, 428.58D0,
     * 432.68D0, 528.99D0, 531.08D0, 628.34D0, 253.24D0, 273.13D0,
     * 273.66D0, 282.10D0, 346.62D0, 347.19D0, 348.78D0, 351.18D0,
     * 450.10D0, 450.35D0, 451.92D0, 455.56D0, 552.22D0, 553.56D0,
     * 555.74D0, 652.59D0, 656.20D0,  14.13D0,  20.41D0,  31.30D0, 
     *  33.84D0,  39.70D0,  48.83D0,  54.50D0,  60.41D0,  72.77D0,
     *  75.25D0,  86.84D0,  94.88D0,  96.40D0, 117.37D0, 139.08D0,
     * 147.73D0, 158.63D0, 161.84D0, 192.11D0, 206.76D0, 209.07D0,
     * 213.32D0, 226.44D0, 237.12D0, 330.90D0, 358.72D0, 370.77D0,
     * 372.72D0, 396.24D0, 416.59D0, 484.02D0, 495.47D0, 514.78D0,
     * 515.65D0, 519.47D0, 544.47D0, 560.11D0, 620.77D0,  18.97D0, 
     *  28.93D0,  33.91D0,  40.03D0,  44.66D0,  49.87D0,  55.16D0, 
     *  60.90D0,  72.08D0,  85.15D0,  97.06D0, 119.63D0, 133.27D0,
     * 143.84D0, 161.91D0, 180.67D0, 198.44D0, 226.86D0, 229.65D0,
     * 258.27D0, 273.77D0, 339.15D0, 350.13D0, 362.75D0, 371.03D0,
     * 393.32D0, 448.53D0, 473.78D0, 511.12D0, 524.70D0, 548.75D0,
     * 551.64D0, 574.02D0, 623.86D0,  21.46D0,  24.33D0,  33.43D0, 
     *  39.22D0,  44.18D0,  55.02D0,  94.33D0,  96.44D0, 118.82D0,
     * 128.48D0, 141.94D0, 156.92D0, 171.65D0, 190.00D0, 223.26D0,
     * 223.88D0, 231.50D0, 265.05D0, 269.44D0, 271.78D0, 273.46D0,
     * 334.61D0, 339.79D0, 349.52D0, 358.18D0, 377.98D0, 394.77D0,
     * 429.66D0, 468.22D0, 487.27D0, 519.54D0, 523.03D0, 612.99D0,
     * 638.59D0, 641.36D0, 622.05D0, 631.50D0, 663.97D0, 646.90D0,
     * 748.29D0, 749.21D0, 750.14D0, 647.04D0, 646.89D0, 746.90D0,
     * 748.43D0, 747.35D0, 749.27D0, 647.61D0, 747.78D0, 750.51D0,
     * 851.37D0, 845.97D0, 847.54D0, 849.93D0, 851.61D0, 849.75D0,
     * 850.98D0, 848.23D0 /
C     y-data 
      DATA ( YDATA(M, 3), M=1, 236) /
     *  0.591D0,  1.547D0,  2.902D0,  2.894D0,  4.703D0,  6.307D0, 
     *  7.03D0,   7.898D0,  9.470D0,  9.484D0, 10.072D0, 10.163D0,
     * 11.615D0, 12.005D0, 12.478D0, 12.982D0, 12.970D0, 13.926D0,
     * 14.452D0, 14.404D0, 15.190D0, 15.550D0, 15.528D0, 15.499D0,
     * 16.131D0, 16.438D0, 16.387D0, 16.549D0, 16.872D0, 16.830D0,
     * 16.926D0, 16.907D0, 16.966D0, 17.060D0, 17.122D0, 17.311D0,
     * 17.355D0, 17.668D0, 17.767D0, 17.803D0, 17.765D0, 17.768D0,
     * 17.736D0, 17.858D0, 17.877D0, 17.912D0, 18.046D0, 18.085D0,
     * 18.291D0, 18.357D0, 18.426D0, 18.584D0, 18.610D0, 18.870D0,
     * 18.795D0, 19.111D0,  0.367D0,  0.796D0,  0.892D0,  1.903D0,
     *  2.150D0,  3.697D0,  5.870D0,  6.421D0,  7.422D0,  9.944D0,
     * 11.023D0, 11.87D0 , 12.786D0, 14.067D0, 13.974D0, 14.462D0,
     * 14.464D0, 15.381D0, 15.483D0, 15.590D0, 16.075D0, 16.347D0,
     * 16.181D0, 16.915D0, 17.003D0, 16.978D0, 17.756D0, 17.808D0,
     * 17.868D0, 18.481D0, 18.486D0, 19.090D0, 16.062D0, 16.337D0,
     * 16.345D0, 16.388D0, 17.159D0, 17.116D0, 17.164D0, 17.123D0,
     * 17.979D0, 17.974D0, 18.007D0, 17.993D0, 18.523D0, 18.669D0,
     * 18.617D0, 19.371D0, 19.330D0, 0.0800D0,  0.248D0,  1.089D0,
     *  1.418D0,  2.278D0,  3.624D0,  4.574D0,  5.556D0,  7.267D0,
     *  7.695D0,  9.136D0,  9.959D0,  9.957D0, 11.600D0, 13.138D0,
     * 13.564D0, 13.871D0, 13.994D0, 14.947D0, 15.473D0, 15.379D0,
     * 15.455D0, 15.908D0, 16.114D0, 17.071D0, 17.135D0, 17.282D0,
     * 17.368D0, 17.483D0, 17.764D0, 18.185D0, 18.271D0, 18.236D0,
     * 18.237D0, 18.523D0, 18.627D0, 18.665D0, 19.086D0,  0.214D0,
     *  0.943D0,  1.429D0,  2.241D0,  2.951D0,  3.782D0,  4.757D0,
     *  5.602D0,  7.169D0,  8.920D0, 10.055D0, 12.035D0, 12.861D0,
     * 13.436D0, 14.167D0, 14.755D0, 15.168D0, 15.651D0, 15.746D0,
     * 16.216D0, 16.445D0, 16.965D0, 17.121D0, 17.206D0, 17.250D0,
     * 17.339D0, 17.793D0, 18.123D0, 18.490D0, 18.566D0, 18.645D0,
     * 18.706D0 , 18.924D0, 19.100D0,  0.375D0,  0.471D0,  1.504D0,
     *  2.204D0,  2.813D0,  4.765D0,  9.835D0, 10.040D0, 11.946D0,
     * 12.596D0, 13.303D0, 13.922D0, 14.440D0, 14.951D0, 15.627D0,
     * 15.639D0, 15.814D0, 16.315D0, 16.334D0, 16.430D0, 16.423D0,
     * 17.024D0, 17.009D0, 17.165D0, 17.134D0, 17.349D0, 17.576D0,
     * 17.848D0, 18.090D0, 18.276D0, 18.404D0, 18.519D0, 19.133D0,
     * 19.074D0, 19.239D0, 19.280D0, 19.101D0, 19.398D0, 19.252D0,
     * 19.890D0, 20.007D0, 19.929D0, 19.268D0, 19.324D0, 20.049D0,
     * 20.107D0, 20.062D0, 20.065D0, 19.286D0, 19.972D0, 20.088D0,
     * 20.743D0, 20.83D0 , 20.935D0, 21.035D0, 20.93D0 , 21.074D0,
     * 21.085D0, 20.935D0 /
C
C                                Problem 4
C     NIST Misra 1d
C     See http://www.itl.nist.gov/div898/strd/nls/data/misra1d.shtml.
C     x-data 
      DATA ( XDATA(M, 4), M=1, 14) /
     *  77.6D0, 114.9D0, 141.1D0, 190.8D0, 239.9D0, 289.0D0, 332.8D0,
     * 378.4D0, 434.8D0, 477.3D0, 536.8D0, 593.1D0, 689.1D0, 760.0D0 /
C     y-data 
      DATA ( YDATA(M, 4), M=1, 14) /
     *  10.07D0, 14.73D0, 17.94D0, 23.93D0, 29.61D0, 35.18D0, 40.02D0,
     *  44.82D0, 50.76D0, 55.05D0, 61.01D0, 66.40D0, 75.47D0, 81.78D0 /
C
C                                Problem 5
C     NIST Lanczos 1
C     See http://www.itl.nist.gov/div898/strd/nls/data/lanczos1.shtml.
C     x-data 
      DATA ( XDATA(M, 5), M=1, 24) /
     * 0.000000000000D+00, 5.000000000000D-02, 1.000000000000D-01,
     * 1.500000000000D-01, 2.000000000000D-01, 2.500000000000D-01,
     * 3.000000000000D-01, 3.500000000000D-01, 4.000000000000D-01,
     * 4.500000000000D-01, 5.000000000000D-01, 5.500000000000D-01,
     * 6.000000000000D-01, 6.500000000000D-01, 7.000000000000D-01,
     * 7.500000000000D-01, 8.000000000000D-01, 8.500000000000D-01,
     * 9.000000000000D-01, 9.500000000000D-01, 1.000000000000D+00,
     * 1.050000000000D+00, 1.100000000000D+00, 1.150000000000D+00 /
C     y-data 
      DATA ( YDATA(M, 5), M=1, 24) /
     * 2.513400000000D+00, 2.044333373291D+00, 1.668404436564D+00,
     * 1.366418021208D+00, 1.123232487372D+00, 9.268897180037D-01,
     * 7.679338563728D-01, 6.388775523106D-01, 5.337835317402D-01,
     * 4.479363617347D-01, 3.775847884350D-01, 3.197393199326D-01,
     * 2.720130773746D-01, 2.324965529032D-01, 1.996589546065D-01,
     * 1.722704126914D-01, 1.493405660168D-01, 1.300700206922D-01,
     * 1.138119324644D-01, 1.000415587559D-01, 8.833209084540D-02,
     * 7.833544019350D-02, 6.976693743449D-02, 6.239312536719D-02 /
C
C                                Problem 6
C     NIST Ratkowksy 2
C     See http://www.itl.nist.gov/div898/strd/nls/data/ratkowsky2.shtml.
C     x-data 
      DATA ( XDATA(M, 6), M=1, 9) /
     *  9.000D0, 14.000D0, 21.000D0, 28.000D0, 42.000D0, 57.000D0,
     * 63.000D0, 70.000D0, 79.000D0 /
C     y-data 
      DATA ( YDATA(M, 6), M=1, 9) /
     *  8.930D0, 10.800D0, 18.590D0, 22.330D0, 39.350D0, 56.110D0,
     * 61.730D0, 64.620D0, 67.080D0 /
C
C                                Problem 7
C     NIST MGH 10
C     See http://www.itl.nist.gov/div898/strd/nls/data/mgh10.shtml.
C     x-data 
      DATA ( XDATA(M, 7), M=1, 16) /
     * 5.000000D+01, 5.500000D+01, 6.000000D+01, 6.500000D+01,
     * 7.000000D+01, 7.500000D+01, 8.000000D+01, 8.500000D+01,
     * 9.000000D+01, 9.500000D+01, 1.000000D+02, 1.050000D+02,
     * 1.100000D+02, 1.150000D+02, 1.200000D+02, 1.250000D+02 /
C    y-data 
      DATA ( YDATA(M, 7), M=1, 16) /
     * 3.478000D+04, 2.861000D+04, 2.365000D+04, 1.963000D+04,
     * 1.637000D+04, 1.372000D+04, 1.154000D+04, 9.744000D+03,
     * 8.261000D+03, 7.030000D+03, 6.005000D+03, 5.147000D+03,
     * 4.427000D+03, 3.820000D+03, 3.307000D+03, 2.872000D+03 /
C
C                                Problem 8
C     NIST Box BOD
C     See http://www.itl.nist.gov/div898/strd/nls/data/boxbod.shtml.
C     x-data 
      DATA ( XDATA(M, 8), M=1, 6) /
     * 1D0, 2D0, 3D0, 5D0, 7D0, 1D1 /
C    y-data 
      DATA ( YDATA(M, 8), M=1, 6) /
     * 1.09D2, 1.49D2, 1.49D2, 1.91D2, 2.13D2, 2.24D2 /
C
C                                Problem 9
C     NIST MGH 17
C     See http://www.itl.nist.gov/div898/strd/nls/data/mgh17.shtml.
C     x-data 
      DATA ( XDATA(M, 9), M=1, 33) /
     * 0.000000D+00, 1.000000D+01, 2.000000D+01, 3.000000D+01,
     * 4.000000D+01, 5.000000D+01, 6.000000D+01, 7.000000D+01,
     * 8.000000D+01, 9.000000D+01, 1.000000D+02, 1.100000D+02,
     * 1.200000D+02, 1.300000D+02, 1.400000D+02, 1.500000D+02,
     * 1.600000D+02, 1.700000D+02, 1.800000D+02, 1.900000D+02,
     * 2.000000D+02, 2.100000D+02, 2.200000D+02, 2.300000D+02,
     * 2.400000D+02, 2.500000D+02, 2.600000D+02, 2.700000D+02,
     * 2.800000D+02, 2.900000D+02, 3.000000D+02, 3.100000D+02,
     * 3.200000D+02 /
C    y-data 
      DATA ( YDATA(M, 9), M=1, 33) /
     * 8.440000D-01, 9.080000D-01, 9.320000D-01, 9.360000D-01,
     * 9.250000D-01, 9.080000D-01, 8.810000D-01, 8.500000D-01,
     * 8.180000D-01, 7.840000D-01, 7.510000D-01, 7.180000D-01,
     * 6.850000D-01, 6.580000D-01, 6.280000D-01, 6.030000D-01,
     * 5.800000D-01, 5.580000D-01, 5.380000D-01, 5.220000D-01,
     * 5.060000D-01, 4.900000D-01, 4.780000D-01, 4.670000D-01,
     * 4.570000D-01, 4.480000D-01, 4.380000D-01, 4.310000D-01,
     * 4.240000D-01, 4.200000D-01, 4.140000D-01, 4.110000D-01,
     * 4.060000D-01 /
C
C                                Problem 10
C     NIST MGH 9
C     See http://www.itl.nist.gov/div898/strd/nls/data/mgh09.shtml.
C     x-data 
      DATA ( XDATA(M, 10), M=1, 11) /
     * 4.00D00, 2.00D00, 1.00D00, 5.00D-1, 2.50D-1, 1.67D-1,
     * 1.25D-1, 1.00D-1, 8.33D-2, 7.14D-2, 6.25D-2 /
C    y-data 
      DATA ( YDATA(M, 10), M=1, 11) /
     * 1.957000D-01, 1.947000D-01, 1.735000D-01, 1.600000D-01,
     * 8.440000D-02, 6.270000D-02, 4.560000D-02, 3.420000D-02,
     * 3.230000D-02, 2.350000D-02, 2.460000D-02 /

      IF (IFLAG .EQ. 1) THEN
        CALL SSQFCN(M, N, X, XDATA(1, IPROB), YDATA(1, IPROB), FVEC, IPROB)
      ELSE
        CALL SSQJAC(M, N, X, XDATA(1, IPROB), YDATA(1, IPROB), FJAC,
     *              LDFJAC, IPROB)
      END IF

      RETURN
      END
C     SUBROUTINE FCN




      SUBROUTINE SSQJAC(M, N, X, XDATA, YDATA, FJAC, LDFJAC, IPROB)

      IMPLICIT NONE

      INTEGER M,N, LDFJAC, IPROB
      DOUBLE PRECISION X(N), XDATA(M), YDATA(M), FJAC(LDFJAC, N)
C     **********
C
C     SUBROUTINE SSQJAC
C
C     THIS SUBROUTINE DEFINES THE JACOBIAN MATRICES OF EIGHTEEN
C     NONLINEAR LEAST SQUARES PROBLEMS. THE PROBLEM DIMENSIONS ARE
C     AS DESCRIBED IN THE PROLOGUE COMMENTS OF SSQFCN.
C
C     THE SUBROUTINE STATEMENT IS
C
C       SUBROUTINE SSQJAC(M,N,X,FJAC,LDFJAC,IPROB)
C
C     WHERE
C
C       M AND N ARE POSITIVE INTEGER INPUT VARIABLES. N MUST NOT
C         EXCEED M.
C
C       X IS AN INPUT ARRAY OF LENGTH N.
C
C       FJAC IS AN M BY N OUTPUT ARRAY WHICH CONTAINS THE JACOBIAN
C         MATRIX OF THE IPROB FUNCTION EVALUATED AT X.
C
C       LDFJAC IS A POSITIVE INTEGER INPUT VARIABLE NOT LESS THAN M
C         WHICH SPECIFIES THE LEADING DIMENSION OF THE ARRAY FJAC.
C
C       IPROB IS A POSITIVE INTEGER VARIABLE WHICH DEFINES THE
C         NUMBER OF THE PROBLEM. IPROB MUST NOT EXCEED 18.
C
C     SUBPROGRAMS CALLED
C
C       FORTRAN-SUPPLIED ... ATAN,COS,EXP,SIN,SQRT
C
C     ARGONNE NATIONAL LABORATORY. MINPACK PROJECT. MARCH 1980.
C     BURTON S. GARBOW, KENNETH E. HILLSTROM, JORGE J. MORE
C
C     **********

      INTEGER I
      DOUBLE PRECISION XX, XX2, XX3, PREFAC, E
C
C     JACOBIAN ROUTINE SELECTOR.
      GO TO ( 100,  200,  300,  400,  500,  600,  700,  800,  900,  1000,
     *       1100, 1200, 1300, 1400), IPROB

C                                Problem 2
C     NIST CHWIRUT2
C     See http://www.itl.nist.gov/div898/strd/nls/data/chwirut2.shtml.
  100 CONTINUE
  
      IF (N .NE. 3) THEN
        WRITE(*, 9980) N
        STOP
      END IF

      DO 110 I = 1, M
        XX = XDATA(I)
        PREFAC = 1D0 / (X(2) + X(3)*XX)
        E = DEXP(-X(1)*XX)
        FJAC(I, 1) = -XX * E * PREFAC
        FJAC(I, 2) = -E * PREFAC**2
        FJAC(I, 3) = -XX * E * PREFAC**2
  110 CONTINUE
      GO TO 9999

C                                Problem 2
C     NIST Misra 1a
C     See http://www.itl.nist.gov/div898/strd/nls/data/misra1a.shtml.
  200 CONTINUE
      DO 210 I = 1, M
        FJAC(I, 1) = 1D0 - DEXP( -X(2) * XDATA(I) )
        FJAC(I, 2) = X(1) * XDATA(I) * DEXP( -X(2) * XDATA(I) )
  210 CONTINUE
      GO TO 9999

C                                Problem 3
C     NIST Hahn 1
C     See http://www.itl.nist.gov/div898/strd/nls/data/hahn1.shtml.
  300 CONTINUE
      DO 310 I = 1, M
        XX = XDATA(I)
        XX2= XX * XX
        XX3= XX2 * XX
        PREFAC= 1D0 / (1D0 + X(5) * XX + X(6) * XX2 + X(7) * XX3)
        FJAC(I, 1)= PREFAC
        FJAC(I, 2)= XX  * PREFAC
        FJAC(I, 3)= XX2 * PREFAC
        FJAC(I, 4)= XX3 * PREFAC
        
        PREFAC= -(X(1) + X(2) * XX + X(3) * XX2 + X(4) * XX3) * PREFAC**2
        FJAC(I, 5)= XX  * PREFAC
        FJAC(I, 6)= XX2 * PREFAC
        FJAC(I, 7)= XX3 * PREFAC
  310 CONTINUE
      GO TO 9999

C                                Problem 4
C     NIST Misra 1d
C     See http://www.itl.nist.gov/div898/strd/nls/data/misra1d.shtml.
  400 CONTINUE
      DO 410 I = 1, M
        XX = 1D0 + X(2) * XDATA(I)
        FJAC(I, 1)= X(2) * XDATA(I) / XX
        FJAC(I, 2)= X(1) * XDATA(I) * (XX - X(2) * XDATA(I)) / XX**2
  410 CONTINUE
      GO TO 9999

C                                Problem 5
C     NIST Lanczos 1
C     See http://www.itl.nist.gov/div898/strd/nls/data/lanczos1.shtml.
  500 CONTINUE
      DO 510 I = 1, M
        FJAC(I, 1)= DEXP(-X(2) * XDATA(I))
        FJAC(I, 2)= -X(1) * XDATA(I) * DEXP(-X(2) * XDATA(I))
        FJAC(I, 3)= DEXP(-X(4) * XDATA(I) )
        FJAC(I, 4)= -X(3) * XDATA(I) * DEXP(-X(4) * XDATA(I))
        FJAC(I, 5)= DEXP(-X(6) * XDATA(I))
        FJAC(I, 6)= -X(5) * XDATA(I) * DEXP(-X(6) * XDATA(I))
  510 CONTINUE
      GO TO 9999

C                                Problem 6
C     NIST Ratkowksy 2
C     See http://www.itl.nist.gov/div898/strd/nls/data/ratkowsky2.shtml.
  600 CONTINUE
      DO 610 I = 1, M
        E= DEXP(X(2) - X(3) * XDATA(I))
        FJAC(I, 1)= 1D0 / (1D0 + E)
        FJAC(I, 2)= -X(1) * E / (1D0 + E)**2
        FJAC(I, 3)=  X(1) * E * XDATA(I) / (1D0 + E)**2
  610 CONTINUE
      GO TO 9999

C                                Problem 7
C     NIST MGH 10
C     See http://www.itl.nist.gov/div898/strd/nls/data/mgh10.shtml.
  700 CONTINUE
      DO 710 I = 1, M
        PREFAC = 1D0 / (xDATA(I) + X(3))
        E= DEXP(X(2) * PREFAC)
        FJAC(I, 1)= E
        FJAC(I, 2)= X(1) * PREFAC * E
        FJAC(I, 3)= -X(2) * X(1) * PREFAC**2 * E
  710 CONTINUE
      GO TO 9999

C                                Problem 8
C     NIST Box BOD
C     See http://www.itl.nist.gov/div898/strd/nls/data/boxbod.shtml.
  800 CONTINUE
      DO 810 I = 1, M
        E= DEXP(-X(2) * XDATA(I))
        FJAC(I, 1)= 1D0 - E
        FJAC(I, 2)= X(1) * XDATA(I) * E
  810 CONTINUE
      GO TO 9999

C                                Problem 9
C     NIST MGH 17
C     See http://www.itl.nist.gov/div898/strd/nls/data/mgh17.shtml.
  900 CONTINUE
      DO 910 I = 1, M
        FJAC(I, 1)= 1D0
        FJAC(I, 2)= DEXP(-X(4) * XDATA(I))
        FJAC(I, 3)= DEXP(-X(5) * XDATA(I))
        FJAC(I, 4)= -XDATA(I) * X(2) * DEXP(-X(4) * XDATA(I))
        FJAC(I, 5)= -XDATA(I) * X(3) * DEXP(-X(5) * XDATA(I))
  910 CONTINUE
      GO TO 9999

C                                Problem 10
C     NIST MGH 9
C     See http://www.itl.nist.gov/div898/strd/nls/data/mgh09.shtml.
 1000 CONTINUE
      DO 1010 I = 1, M
        XX    = XDATA(I)
        xx2   = XX**2
        PREFAC= 1D0 / (XX2 + XX * X(3) + X(4))
        FJAC(I, 1)= (XX2 + XX * X(2)) * PREFAC
        FJAC(I, 2)= X(1) * XX * PREFAC
        FJAC(I, 3)= -X(1) * (XX2 + XX * X(2)) * XX * PREFAC**2
        FJAC(I, 4)= -X(1) * (XX2 + XX * X(2)) * PREFAC**2
 1010 CONTINUE
      GO TO 9999

 1100 CONTINUE
      GO TO 9999

 1200 CONTINUE
      GO TO 9999

 1300 CONTINUE
      GO TO 9999

 1400 CONTINUE
      GO TO 9999

 9980 FORMAT ('Expected ', I3, 'unknows, found ', I3)

 9999 CONTINUE
      RETURN
C
C     LAST CARD OF SUBROUTINE SSQJAC.
C
      END




      SUBROUTINE INITPT(N, M, X, IPROB, TRY, FACTOR, FTOL, XTOL, GTOL,
     *                  MAXFEV, MODE, PNAME)

      IMPLICIT NONE

      INTEGER N, M, IPROB, TRY, MAXFEV, MODE
      DOUBLE PRECISION FACTOR, FTOL, XTOL, GTOL

      CHARACTER*64 PNAME
      
      DOUBLE PRECISION X(N)

C     **********
C
C     SUBROUTINE INITPT
C
c     This subroutine specifies the parameters  and starting points for
C     the functions defined by subroutine ssqfcn.
C
C     THE SUBROUTINE STATEMENT IS
C
C       SUBROUTINE INITPT(N,X,IPROB,FACTOR)
C
C     WHERE
C
C       N IS A POSITIVE INTEGER INPUT VARIABLE.
C
C       X IS AN OUTPUT ARRAY OF LENGTH N WHICH CONTAINS THE STANDARD
C         STARTING POINT FOR PROBLEM IPROB MULTIPLIED BY FACTOR.
C
C       IPROB IS A POSITIVE INTEGER INPUT VARIABLE WHICH DEFINES THE
C         NUMBER OF THE PROBLEM. IPROB MUST NOT EXCEED 18.
C
C       FACTOR IS AN INPUT VARIABLE WHICH SPECIFIES THE MULTIPLE OF
C         THE STANDARD STARTING POINT. IF FACTOR IS UNITY, NO
C         MULTIPLICATION IS PERFORMED.
C
C     ARGONNE NATIONAL LABORATORY. MINPACK PROJECT. MARCH 1980.
C     BURTON S. GARBOW, KENNETH E. HILLSTROM, JORGE J. MORE
C
C     **********

C     Set default values for each problem

      DOUBLE PRECISION DPMPAR

      N= 0
      M= 0
      FACTOR= 100D0
      FTOL= DSQRT( DPMPAR(1) )
      XTOL= DSQRT( DPMPAR(1) )
      GTOL= 0D0
      MAXFEV= 400
      MODE= 1
      PNAME= ""

C
C     SELECTION OF INITIAL POINT.
      GO TO ( 100,  200,  300,  400,  500,  600,  700,  800,  900,  1000,
     *       1100, 1200, 1300, 1400), IPROB

C                                Problem 1
C     NIST CHWIRUT2
C     See http://www.itl.nist.gov/div898/strd/nls/data/chwirut2.shtml.
  100 CONTINUE
      PNAME="CHWIRUT 2"
      M= 54
      N= 3

      IF ( TRY.EQ.1 ) THEN
        X(1) = 1D-1
        X(2) = 1D-2
        X(3) = 2D-2
      ELSE IF ( TRY.EQ.2 ) THEN
        X(1) = 1.5D-1
        X(2) = 8D-3
        X(3) = 1D-2
        FTOL= 1D6 * DPMPAR(1)
        XTOL= 1D6 * DPMPAR(1)
      ELSE
        N=0
      END IF
      GO TO 9999
C                                Problem 2
C     NIST Misra 1a
C     See http://www.itl.nist.gov/div898/strd/nls/data/misra1a.shtml.
  200 CONTINUE
      PNAME="MISRA 1A"
      M= 14
      N= 2

      IF ( TRY.EQ.1 ) THEN
        X(1) = 500D0
        X(2) = 1D-4
      ELSE IF ( TRY.EQ.2 ) THEN
        X(1) = 250D0
        X(2) = 5D-4
      ELSE
        N=0
      END IF
      GO TO 9999

C                                Problem 3
C     NIST Hahn 1
C     See http://www.itl.nist.gov/div898/strd/nls/data/hahn1.shtml.
  300 CONTINUE
      PNAME="HAHN 1"
      M= 236
      N= 7

      IF ( TRY.EQ.1 ) THEN
        X(1) =  1D1
        X(2) = -1D0
        X(3) =  5D-2
        X(4) = -1D-5
        X(5) = -5D-2
        X(6) =  1D-3
        X(7) = -1D-6
      ELSE IF ( TRY.EQ.2 ) THEN
        X(1) =  1D-1
        X(2) = -1D-1
        X(3) =  5D-3
        X(4) = -1D-6
        X(5) = -5D-3
        X(6) =  1D-4
        X(7) = -1D-7
      ELSE
        N=0
      END IF
      GO TO 9999

C                                Problem 4
C     NIST Misra 1d
C     See http://www.itl.nist.gov/div898/strd/nls/data/misra1d.shtml.
  400 CONTINUE
      PNAME="MISRA 1D"
      M= 14
      N= 2

      IF ( TRY.EQ.1 ) THEN
        X(1) = 5D2
        X(2) = 1D-4
      ELSE IF ( TRY.EQ.2 ) THEN
        X(1) = 45D1
        X(2) = 3D-4
      ELSE
        N=0
      END IF
      GO TO 9999

C                                Problem 5
C     NIST Lanczos 1
C     See http://www.itl.nist.gov/div898/strd/nls/data/lanczos1.shtml.
  500 CONTINUE
      PNAME="LANCZOS 1"
      M= 24
      N= 6

      IF ( TRY.EQ.1 ) THEN
        X(1) = 1.2D0
        X(2) = 3D-1
        X(3) = 5.6D0
        X(4) = 5.5D0
        X(5) = 6.5D0
        X(6) = 7.6D0
      ELSE IF ( TRY.EQ.2 ) THEN
        X(1) = 5D-1
        X(2) = 7D-1
        X(3) = 3.6D0
        X(4) = 4.2D0
        X(5) = 4D0
        X(6) = 6.3D0
      ELSE
        N=0
      END IF
      GO TO 9999

C                                Problem 6
C     NIST Ratkowksy 2
C     See http://www.itl.nist.gov/div898/strd/nls/data/ratkowsky2.shtml.
  600 CONTINUE
      PNAME="RATKOWSKY 2"
      M= 9
      N= 3

      IF ( TRY.EQ.1 ) THEN
        X(1) = 1D2
        X(2) = 1D0
        X(3) = 1D-1
      ELSE IF ( TRY.EQ.2 ) THEN
        X(1) = 7.5D1
        X(2) = 2.5D0
        X(3) = 7D-2
      ELSE
        N=0
      END IF
      GO TO 9999

C                                Problem 7
C     NIST MGH 10
C     See http://www.itl.nist.gov/div898/strd/nls/data/mgh10.shtml.
  700 CONTINUE
      PNAME="MGH 10"
      M= 16
      N= 3

      IF ( TRY.EQ.1 ) THEN
        X(1) = 2D0
        X(2) = 4D5
        X(3) = 2.5D4
      ELSE IF ( TRY.EQ.2 ) THEN
        X(1) = 2D-2
        X(2) = 4D3
        X(3) = 2.5D2
      ELSE
        N=0
      END IF
      GO TO 9999

C                                Problem 8
C     NIST Box BOD
C     See http://www.itl.nist.gov/div898/strd/nls/data/boxbod.shtml.
  800 CONTINUE
      PNAME="BOX BOD"
      M= 6
      N= 2

      IF ( TRY.EQ.1 ) THEN
        FTOL  = 1D6 * DPMPAR(1)
        XTOL  = 1D6 * DPMPAR(1)
        FACTOR= 1D1

        X(1) = 1D0
        X(2) = 1D0
      ELSE IF ( TRY.EQ.2 ) THEN
        FTOL  = DPMPAR(1)
        XTOL  = DPMPAR(1)

        X(1) = 1D2
        X(2) = 7.5D-1
      ELSE
        N=0
      END IF
      GO TO 9999

C                                Problem 9
C     NIST MGH 17
C     See http://www.itl.nist.gov/div898/strd/nls/data/mgh17.shtml.
  900 CONTINUE
      PNAME="MGH 17"
      M= 33
      N= 5

      IF ( TRY.EQ.1 ) THEN
        FTOL  = DPMPAR(1)
        XTOL  = DPMPAR(1)
        MAXFEV= 1000

        X(1) = 5D1
        X(2) = 1.5D2
        X(3) = -1D2
        X(4) = 1D0
        X(5) = 2D0
      ELSE IF ( TRY.EQ.2 ) THEN
        X(1) = 5D-1
        X(2) = 1.5D0
        X(3) = -1D0
        X(4) = 1D-2
        X(5) = 2D-2
      ELSE
        N=0
      END IF
      GO TO 9999

C                                Problem 10
C     NIST MGH 9
C     See http://www.itl.nist.gov/div898/strd/nls/data/mgh09.shtml.
 1000 CONTINUE
      PNAME="MGH 9"
      M= 11
      N= 4

      IF ( TRY.EQ.1 ) THEN
        MAXFEV= 1000

        X(1) = 25D0
        X(2) = 39D0
        X(3) = 41.5D0
        X(4) = 39D0
      ELSE IF ( TRY.EQ.2 ) THEN
        X(1) = 25D-2
        X(2) = 39D-2
        X(3) = 41.5D-2
        X(4) = 39D-2
      ELSE
        N=0
      END IF
      GO TO 9999

 1100 CONTINUE
      GO TO 9999

 1200 CONTINUE
      GO TO 9999

 1300 CONTINUE
      GO TO 9999

 1400 CONTINUE
      GO TO 9999

 9999 CONTINUE
      RETURN
      END
C     SUBROUTINE INITPT.



      SUBROUTINE SSQFCN(M, N, X, XDATA, YDATA, FVEC, IPROB)

      IMPLICIT NONE

      INTEGER          M, N, IPROB
      DOUBLE PRECISION X(N), XDATA(M), YDATA(M), FVEC(M)
C     **********
C
C     SUBROUTINE SSQFCN
C
C     THIS SUBROUTINE DEFINES THE FUNCTIONS OF EIGHTEEN NONLINEAR
C     LEAST SQUARES PROBLEMS. THE ALLOWABLE VALUES OF (M,N) FOR
C     FUNCTIONS 1, 2 AND 3 ARE VARIABLE BUT WITH M .GE. N.
C     FOR FUNCTIONS 4, 5, 6, 7, 8, 9 AND 10 THE VALUES OF (M,N) ARE
C     (2, 2),(3, 3),(4, 4),(2, 2),(15, 3),(11, 4) AND (16, 3),
C     RESPECTIVELY.
C     FUNCTION 11 (WATSON) HAS M = 31 WITH N USUALLY 6 OR 9.
C     HOWEVER, ANY N, N = 2,..., 31, IS PERMITTED.
C     FUNCTIONS 12, 13 AND 14 HAVE N = 3, 2 AND 4, RESPECTIVELY, BUT
C     ALLOW ANY M .GE. N, WITH THE USUAL CHOICES BEING 10, 10 AND 20.
C     FUNCTION 15 (CHEBYQUAD) ALLOWS M AND N VARIABLE WITH M .GE. N.
C     FUNCTION 16 (BROWN) ALLOWS N VARIABLE WITH M = N.
C     FOR FUNCTIONS 17 AND 18, THE VALUES OF (M,N) ARE
C     (33, 5) AND (65, 11), RESPECTIVELY.
C
C     THE SUBROUTINE STATEMENT IS
C
C       SUBROUTINE SSQFCN(M,N,X,FVEC,IPROB)
C
C     WHERE
C
C       M AND N ARE POSITIVE INTEGER INPUT VARIABLES. N MUST NOT
C         EXCEED M.
C
C       X IS AN INPUT ARRAY OF LENGTH N.
C
C       FVEC IS AN OUTPUT ARRAY OF LENGTH M WHICH CONTAINS THE IPROB
C         FUNCTION EVALUATED AT X.
C
C       IPROB IS A POSITIVE INTEGER INPUT VARIABLE WHICH DEFINES THE
C         NUMBER OF THE PROBLEM. IPROB MUST NOT EXCEED 18.
C
C     SUBPROGRAMS CALLED
C
C       FORTRAN-SUPPLIED ... ATAN,COS,EXP,SIN,SQRT,SIGN
C
C     ARGONNE NATIONAL LABORATORY. MINPACK PROJECT. MARCH 1980.
C     BURTON S. GARBOW, KENNETH E. HILLSTROM, JORGE J. MORE
C
C     **********
      INTEGER I

      DOUBLE PRECISION XX, XX2, XX3
C
C     FUNCTION ROUTINE SELECTOR.
C
      GO TO ( 100,  200,  300,  400,  500,  600,  700,  800,  900,  1000,
     *       1100, 1200, 1300, 1400), IPROB


c                                Problem 1
C     NIST CHWIRUT2
C     See http://www.itl.nist.gov/div898/strd/nls/data/chwirut2.shtml.
  100 CONTINUE

      DO  110  I = 1, M
        XX= XDATA(I)
        FVEC(I)= DEXP(-X(1)*XX) / (X(2) + X(3)*XX) - YDATA(I)
  110 CONTINUE
      GO TO 9999
C                                Problem 2
C     NIST Misra 1a
C     See http://www.itl.nist.gov/div898/strd/nls/data/misra1a.shtml.
  200 CONTINUE
      DO  210  I = 1, M
        FVEC(I) = X(1) * (1D0 - DEXP(-X(2) * XDATA(I))) - YDATA(I)
  210 CONTINUE
      GO TO 9999
C                                Problem 3
C     NIST Hahn 1
C     See http://www.itl.nist.gov/div898/strd/nls/data/hahn1.shtml.
  300 CONTINUE
      DO  310  I = 1, M
        XX =XDATA(I)
        XX2= XX * XX
        XX3= XX2 * xX
        FVEC(I)= (X(1) + X(2) * XX + X(3) * XX2 + X(4) * XX3)
     *         / ( 1D0 + X(5) * XX + X(6) * XX2 + X(7) * XX3) - YDATA(I)
  310 CONTINUE
      GO TO 9999

C                                Problem 4
C     NIST Misra 1d
C     See http://www.itl.nist.gov/div898/strd/nls/data/misra1d.shtml.
  400 CONTINUE
      DO  410  I = 1, M
        XX= XDATA(I)
        FVEC(I)= X(1) * X(2) * XX / (1D0 + X(2) * XX) - YDATA(I)
  410 CONTINUE
      GO TO 9999

C                                Problem 5
C     NIST Lanczos 1
C     See http://www.itl.nist.gov/div898/strd/nls/data/lanczos1.shtml.
  500 CONTINUE
      DO  510  I = 1, M
        XX= XDATA(I)
        FVEC(I) = X(1) * DEXP(-X(2) * XX) + X(3) * DEXP(-X(4) * XX)
     *          + X(5) * DEXP(-X(6) * XX) - YDATA(I)
  510 CONTINUE
      GO TO 9999

C                                Problem 6
C     NIST Ratkowksy 2
C     See http://www.itl.nist.gov/div898/strd/nls/data/ratkowsky2.shtml.
  600 CONTINUE
      DO  610  I = 1, M
        FVEC(I)= X(1) / (1D0 + DEXP(X(2) - X(3) * XDATA(I))) - YDATA(I)
  610 CONTINUE
      GO TO 9999

C                                Problem 7
C     NIST MGH 10
C     See http://www.itl.nist.gov/div898/strd/nls/data/mgh10.shtml.
  700 CONTINUE
      DO  710  I = 1, M
        FVEC(I)= X(1) * DEXP(X(2) / (XDATA(I) + X(3))) - YDATA(I)
  710 CONTINUE
      GO TO 9999

C                                Problem 8
C     NIST Box BOD
C     See http://www.itl.nist.gov/div898/strd/nls/data/boxbod.shtml.
  800 CONTINUE
      DO  810  I = 1, M
        FVEC(I)= X(1) * (1D0 - DEXP(-X(2) * XDATA(I))) - YDATA(I)
  810 CONTINUE
      GO TO 9999

C                                Problem 9
C     NIST MGH 17
C     See http://www.itl.nist.gov/div898/strd/nls/data/mgh17.shtml.
  900 CONTINUE
      DO  910  I = 1, M
        FVEC(I)= X(1) + X(2) * DEXP(-X(4) * xDATA(I))
     *                + X(3) * DEXP(-X(5) * xDATA(I)) - YDATA(I)
  910 CONTINUE
      GO TO 9999

C                                Problem 10
C     NIST MGH 9
C     See http://www.itl.nist.gov/div898/strd/nls/data/mgh09.shtml.
 1000 CONTINUE
      DO  1010  I = 1, M
        XX= XDATA(I)
        XX2= XX**2
        FVEC(I)= X(1) * (xx2 + XX * X(2)) / (XX2 + XX * X(3) + X(4))
     *         - YDATA(I)
 1010 CONTINUE
      GO TO 9999

 1100 CONTINUE
      GO TO 9999

 1200 CONTINUE
      GO TO 9999

 1300 CONTINUE
      GO TO 9999

 1400 CONTINUE
      GO TO 9999

 9999 CONTINUE

      RETURN
      END
C     SUBROUTINE SSQFCN.
